################################################################
##
## R script for calculating decomposition of changes in completed cohort fertility rates into the effect of parity progression ratios for paper:
## "Cohort fertility decline in low fertility countries: decomposition using parity progression ratios"
## by Krystof Zeman, Eva Beaujouan, Zuzanna Brzozowska and Tomas Sobotka
## in Demographic Research
##
## Wittgenstein Centre for Demography and Global Human Capital (IIASA, VID/AW, WU), Vienna Institute of Demography/Austrian Academy of Sciences
## November 2017
##
################################################################

library(zoo) ## for linear interpolation

## all data files are located in the zipped folder "data"
## most of the data come from the CFE database at: http://www.cfe-database.org/
## some data are reconstructed from Human Fertility Database data (see Supplement table 1 for details)
## the script should work also for other data from CFE database, not included in the paper
DATAFILES<-list.files("data") 

## file "countries.csv" contains some metainformation on the data
COUNTRIES<-read.csv("countries.csv", header = TRUE, as.is=TRUE)

## the 8 regions used for the paper are as:
## 1=Eastern Europe
## 2=Central Europe
## 3=Western Europe
## 4=Northern Europe
## 5=German-speaking
## 6=Southern Europe
## 7=English-speaking non-European
## 8=East Asia

FIRSTYEAR<-1940
LASTYEAR<-1970
LASTAGE<-43
COHORTS<-FIRSTYEAR:LASTYEAR

DATAALL<-NULL
DATAALLreduced<-NULL


#### computation for each country ####
for (DATAFILE in DATAFILES){
  
  #### read data ####
  message(DATAFILE)
  COUNTRY_YEAR<-substr(DATAFILE,1,8)
  YEAR<-as.numeric(substr(DATAFILE,5,8))
  COUNTRYCODE<-substr(DATAFILE,1,3)
  REGION<-COUNTRIES$REGION[COUNTRIES$FILE==DATAFILE]
    
  DATAFILENAME<-paste0("data/",DATAFILE)
  DATA<-read.csv(DATAFILENAME,header=TRUE,as.is=TRUE)
  
  COUNTRYNAME<-unique(DATA$country)
  SURVEY<-unique(DATA$data_source)

  #### summarising parity distribution ####
  DATA<-DATA[DATA$sex==FALSE|DATA$sex=="F",]
  DATA<-DATA[DATA$cohort_from>=FIRSTYEAR,]
  DATA<-DATA[DATA$cohort_from<=YEAR-LASTAGE,]

  DATA1<-aggregate(value~cohort_from+stat,DATA,sum)
  NCOL=dim(DATA1)[2]

  DATA1<-subset(DATA1,DATA1$cohort_from!="Unknown")
  DATA2<-subset(DATA1,DATA1$stat=="children_total")
  DATA3<-subset(DATA1,DATA1$stat=="parity_0")
  DATA4<-subset(DATA1,DATA1$stat=="parity_1")
  DATA5<-subset(DATA1,DATA1$stat=="parity_2")
  DATA6<-subset(DATA1,DATA1$stat=="parity_3")
  DATA7<-subset(DATA1,DATA1$stat=="parity_4")
  DATA8<-subset(DATA1,DATA1$stat=="parity_unknown")
  DATA9<-subset(DATA1,DATA1$stat=="women_total")
  colnames(DATA2)[NCOL]<-"children_total"
  colnames(DATA3)[NCOL]<-"parity_0"
  colnames(DATA4)[NCOL]<-"parity_1"
  colnames(DATA5)[NCOL]<-"parity_2"
  colnames(DATA6)[NCOL]<-"parity_3"
  colnames(DATA7)[NCOL]<-"parity_4"
  colnames(DATA8)[NCOL]<-"parity_unknown"
  colnames(DATA9)[NCOL]<-"women_total"
  DATA2$stat<-NULL
  DATA3$stat<-NULL
  DATA4$stat<-NULL
  DATA5$stat<-NULL
  DATA6$stat<-NULL
  DATA7$stat<-NULL
  DATA8$stat<-NULL
  DATA9$stat<-NULL
  
  DATA1<-merge(DATA9,DATA2,all.x=T,all.y=F)
  DATA1<-merge(DATA1,DATA3,all.x=T,all.y=F)
  DATA1<-merge(DATA1,DATA4,all.x=T,all.y=F)
  DATA1<-merge(DATA1,DATA5,all.x=T,all.y=F)
  DATA1<-merge(DATA1,DATA6,all.x=T,all.y=F)
  DATA1<-merge(DATA1,DATA7,all.x=T,all.y=F)
  DATA1<-merge(DATA1,DATA8,all.x=T,all.y=F)
  
  #### linear interpolation of data which are available only for 5-year age groups (Italy and France) ####
  DATA1[,(NCOL-1):(NCOL+6)][is.na(DATA1[,(NCOL-1):(NCOL+6)])]<-0
  COHORTS1<-min(DATA1$cohort_from):max(DATA1$cohort_from)
  if(length(DATA1$cohort_from)!=length(COHORTS1)){
    DATA1C<-data.frame(COHORTS1)
    colnames(DATA1C)<-"cohort_from"
    DATA1<-merge(DATA1C,DATA1,all=T)
    DATA1[,2:9]<-na.approx(DATA1[,2:9])
  }

  #### calculating of completed cohort fertility (CFR) by birth order ####
  DATA1$CFR<-(DATA1$children_total)/
    (DATA1$women_total-DATA1$parity_unknown)
  DATA1$CFR1<-(DATA1$women_total-DATA1$parity_unknown-DATA1$parity_0)/
    (DATA1$women_total-DATA1$parity_unknown)
  DATA1$CFR2<-(DATA1$women_total-DATA1$parity_unknown-DATA1$parity_0-DATA1$parity_1)/
    (DATA1$women_total-DATA1$parity_unknown)
  DATA1$CFR3<-(DATA1$women_total-DATA1$parity_unknown-DATA1$parity_0-DATA1$parity_1-DATA1$parity_2)/
    (DATA1$women_total-DATA1$parity_unknown)
  DATA1$CFR4<-DATA1$CFR-DATA1$CFR1-DATA1$CFR2-DATA1$CFR3 ## CFR4+
  
  
  #### calculating of parity progression ratios (PPR) ####
  DATA1$PPR01<-DATA1$CFR1
  DATA1$PPR12<-DATA1$CFR2/DATA1$CFR1
  DATA1$PPR23<-DATA1$CFR3/DATA1$CFR2
  DATA1$PPR34<-DATA1$CFR4/(DATA1$CFR3+DATA1$CFR4) ## PPR3+4+=CFR4+/CFR3+
  
  #### calculating of changes in CFR (dCFR) ####
  DATA1$dCFR <-DATA1$CFR-DATA1$CFR[1]
  DATA1$dCFR1<-DATA1$CFR1-DATA1$CFR1[1]
  DATA1$dCFR2<-DATA1$CFR2-DATA1$CFR2[1]
  DATA1$dCFR3<-DATA1$CFR3-DATA1$CFR3[1]
  DATA1$dCFR4<-DATA1$CFR4-DATA1$CFR4[1] ## dCFR4+
  
  #### decomposition ####
  DATA1$CFR_fix1234<-DATA1$PPR01 + DATA1$PPR01*DATA1$PPR12[1] + DATA1$PPR01*DATA1$PPR12[1]*DATA1$PPR23[1] + DATA1$PPR01*DATA1$PPR12[1]*DATA1$PPR23[1]*DATA1$PPR34[1]/(1-DATA1$PPR34[1])
  DATA1$CFR_fix234 <-DATA1$PPR01 + DATA1$PPR01*DATA1$PPR12    + DATA1$PPR01*DATA1$PPR12   *DATA1$PPR23[1] + DATA1$PPR01*DATA1$PPR12   *DATA1$PPR23[1]*DATA1$PPR34[1]/(1-DATA1$PPR34[1])
  DATA1$CFR_fix34  <-DATA1$PPR01 + DATA1$PPR01*DATA1$PPR12    + DATA1$PPR01*DATA1$PPR12   *DATA1$PPR23    + DATA1$PPR01*DATA1$PPR12   *DATA1$PPR23   *DATA1$PPR34[1]/(1-DATA1$PPR34[1])
  
  DATA1$dPPR01<-DATA1$CFR_fix1234-DATA1$CFR[1]
  DATA1$dPPR12<-DATA1$CFR_fix234-DATA1$CFR_fix1234
  DATA1$dPPR23<-DATA1$CFR-DATA1$CFR_fix234 ## dPPR23+

  #### parity distribution ####
  DATA1$prop0<-DATA1$parity_0/(DATA1$women_total-DATA1$parity_unknown)
  DATA1$prop1<-DATA1$parity_1/(DATA1$women_total-DATA1$parity_unknown)
  DATA1$prop2<-DATA1$parity_2/(DATA1$women_total-DATA1$parity_unknown)
  DATA1$prop3<-1-DATA1$prop0-DATA1$prop1-DATA1$prop2 ## prop3+
  
  #### constructing table ####
  DATAALL1<-data.frame(COUNTRYCODE,REGION,DATA1)
  colnames(DATAALL1)[1:2]<-c("countrycode","region")
  DATAALL1<-DATAALL1[is.na(DATAALL1$women_total)==F,]
  DATAALL<-rbind(DATAALL,DATAALL1)
  
  #### reduced table (just years 1940, 1955, 1970)
  DATA1940<-DATAALL1[DATAALL1$cohort_from==min(DATAALL1$cohort_from),]  
  DATA1955<-DATAALL1[DATAALL1$cohort_from==min(DATAALL1$cohort_from[DATAALL1$cohort_from>=1955]),]  
  DATA1970<-DATAALL1[DATAALL1$cohort_from==max(DATAALL1$cohort_from),]  
  DATAALLreduced<-rbind(DATAALLreduced,DATA1940,DATA1955,DATA1970)
}


#### save data ####
write.csv(DATAALL,file="decomposition_data.csv",row.names=F,quote=F)
write.csv(DATAALLreduced,file="decomposition_data_reduced.csv",row.names=F,quote=F)


#### aggregate means of indicators for regions ####
DATAALLreduced$cohort_from2<-c(1940,1955,1970)
TABLE<-aggregate(DATAALLreduced[,12:35],
              list(DATAALLreduced$region,
                   DATAALLreduced$cohort_from2),mean)
DATAALLreduced$cohort_from<-as.numeric(DATAALLreduced$cohort_from)
COHORT_MIN<-aggregate(DATAALLreduced$cohort_from,
                      list(DATAALLreduced$region,
                           DATAALLreduced$cohort_from2),min)
COHORT_MAX<-aggregate(DATAALLreduced$cohort_from,
                      list(DATAALLreduced$region,
                           DATAALLreduced$cohort_from2),max)
TABLE[1:8,2]<-COHORT_MAX[1:8,3]
TABLE[17:24,2]<-COHORT_MIN[17:24,3]
colnames(TABLE)[1:2]<-c("region","cohort")
write.csv(TABLE,file="decomposition_data_regions.csv",row.names=F,quote=F)

#### FINITO ####